@@ -2,31 +2,18 @@
 	Cuhre.c
 		Adaptive integration using cubature rules
 		by Thomas Hahn
-		last modified 16 Jun 10 th
+		last modified 19 Dec 11 th
 */
 
 
+#define CUHRE
+#define ROUTINE "Cuhre"
+
 #include "decl.h"
-
-#define Print(s) puts(s); fflush(stdout)
+#include "CSample.c"
 
 /*********************************************************************/
 
-static inline void DoSample(This *t, count n, creal *x, real *f)
-{
-  t->neval += n;
-  while( n-- ) {
-    if( t->integrand(&t->ndim, x, &t->ncomp, f, t->userdata) == ABORT )
-      longjmp(t->abort, 1);
-    x += t->ndim;
-    f += t->ncomp;
-  }
-}
-
-/*********************************************************************/
-
-#include "common.c"
-
 Extern void EXPORT(Cuhre)(ccount ndim, ccount ncomp,
   Integrand integrand, void *userdata,
   creal epsrel, creal epsabs,
