@@ -2,7 +2,7 @@
 	Integrate.c
 		integrate over the unit hypercube
 		this file is part of Cuhre
-		last modified 7 Jun 10 th
+		last modified 28 Mar 13 th
 */
 
 
@@ -17,7 +17,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   } Pool;
 
   count dim, comp, ncur, ipool, npool;
-  int fail = -99;
+  int fail;
   Totals totals[NCOMP];
   Pool *cur = NULL, *pool;
   Region *region;
@@ -39,11 +39,14 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   if( BadComponent(t) ) return -2;
   if( BadDimension(t) ) return -1;
 
-  RuleAlloc(t);
   t->epsabs = Max(t->epsabs, NOTZERO);
-  t->mineval = IMax(t->mineval, t->rule.n + 1);
 
-  if( setjmp(t->abort) ) goto abort;
+  RuleAlloc(t);
+  t->mineval = IMax(t->mineval, t->rule.n + 1);
+  FrameAlloc(t, ShmRm(t));
+  ForkCores(t);
+
+  if( (fail = setjmp(t->abort)) ) goto abort;
 
   Alloc(cur, 1);
   cur->next = NULL;
@@ -103,13 +106,13 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       }
     }
 
-    if( maxratio <= 1 && t->neval >= t->mineval ) {
-      fail = 0;
+    if( maxratio <= 1 && t->neval >= t->mineval ) break;
+
+    if( t->neval >= t->maxeval ) {
+      fail = 1;
       break;
     }
 
-    if( t->neval >= t->maxeval ) break;
-
     maxerr = -INFTY;
     regionL = cur->region;
     npool = ncur;
@@ -132,8 +135,8 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     regionR = &cur->region[ncur++];
 
     regionR->div = ++regionL->div;
-    ResCopy(result, regionL->result);
-    VecCopy(regionR->bounds, regionL->bounds);
+    FCopy(result, regionL->result);
+    XCopy(regionR->bounds, regionL->bounds);
 
     bisectdim = result[maxcomp].bisectdim;
     bL = &regionL->bounds[bisectdim];
@@ -163,7 +166,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       rR->err += diff;
       tot->lasterr += rL->err + rR->err - r->err;
 
-      tot->weightsum += w = 1/Max(Sq(tot->lasterr), NOTZERO);
+      tot->weightsum += w = 1/Max(tot->lasterr, NOTZERO);
       sigsq = 1/tot->weightsum;
       tot->avgsum += w*tot->lastavg;
       avg = sigsq*tot->avgsum;
@@ -226,6 +229,8 @@ abort:
     free(pool);
   }
 
+  WaitCores(t);
+  FrameFree(t);
   RuleFree(t);
 
   return fail;
