@@ -2,13 +2,12 @@
 	decl.h
 		Type declarations
 		this file is part of Cuhre
-		last modified 19 Jan 05 th
+		last modified 21 Dec 11 th
 */
 
 
 #include "stddecl.h"
 
-
 typedef struct {
   real avg, err;
   count bisectdim;
@@ -16,7 +15,6 @@ typedef struct {
 
 typedef const Result cResult;
 
-
 typedef struct {
   real avg, err, lastavg, lasterr;
   real weightsum, avgsum;
@@ -25,16 +23,13 @@ typedef struct {
 
 typedef const Totals cTotals;
 
-
 typedef struct {
   real lower, upper;
 } Bounds;
 
 typedef const Bounds cBounds;
 
-
 typedef struct {
-  real *x, *f;
   void *first, *last;
   real errcoeff[3];
   count n;
@@ -42,15 +37,36 @@ typedef struct {
 
 typedef const Rule cRule;
 
+typedef int (*Integrand)(ccount *, creal *, ccount *, real *, void *);
+
+typedef struct _this {
+  count ndim, ncomp;
+#ifndef MLVERSION
+  Integrand integrand;
+  void *userdata;
+#ifdef HAVE_FORK
+  int ncores, *child;
+  SHM_ONLY(int shmid;)
+#endif
+#endif
+  real *frame;
+  real epsrel, epsabs;
+  int flags;
+  number mineval, maxeval;
+  count key, nregions;
+  number neval;
+  Rule rule;
+  jmp_buf abort;
+} This;
+
+#define nframe rule.n
+
+typedef const This cThis;
 
 #define TYPEDEFREGION \
   typedef struct region { \
-    struct region *next; \
     count div; \
     Result result[NCOMP]; \
     Bounds bounds[NDIM]; \
   } Region
 
-
-typedef void (*Integrand)(ccount *, creal *, ccount *, real *);
-
