@@ -4,63 +4,17 @@
 		originally by J.H. Friedman and M.H. Wright
 		(CERNLIB subroutine D151)
 		this version by Thomas Hahn
-		last modified 15 Feb 11 th
+		last modified 19 Dec 11 th
 */
 
+#define DIVONNE
+#define ROUTINE "Divonne"
+
 #include "decl.h"
-
-#define Print(s) puts(s); fflush(stdout)
+#include "CSample.c"
 
 /*********************************************************************/
 
-static inline void DoSample(This *t, number n, ccount ldx, creal *x, real *f)
-{
-  t->neval += n;
-  while( n-- ) {
-    if( t->integrand(&t->ndim, x, &t->ncomp, f, t->userdata, &t->phase) == ABORT )
-      longjmp(t->abort, -99);
-    x += ldx;
-    f += t->ncomp;
-  }
-}
-
-/*********************************************************************/
-
-static inline count SampleExtra(This *t, cBounds *b)
-{
-  number n = t->nextra;
-  t->peakfinder(&t->ndim, b, &n, t->xextra);
-  DoSample(t, n, t->ldxgiven, t->xextra, t->fextra);
-  return n;
-}
-
-/*********************************************************************/
-
-static inline void AllocGiven(This *t, creal *xgiven)
-{
-  if( t->ngiven | t->nextra ) {
-    cnumber nxgiven = t->ngiven*(t->ldxgiven = IMax(t->ldxgiven, t->ndim));
-    cnumber nxextra = t->nextra*t->ldxgiven;
-    cnumber nfgiven = t->ngiven*t->ncomp;
-    cnumber nfextra = t->nextra*t->ncomp;
-
-    Alloc(t->xgiven, nxgiven + nxextra + nfgiven + nfextra);
-    t->xextra = t->xgiven + nxgiven;
-    t->fgiven = t->xextra + nxextra;
-    t->fextra = t->fgiven + nfgiven;
-
-    if( nxgiven ) {
-      t->phase = 0;
-      Copy(t->xgiven, xgiven, nxgiven);
-      DoSample(t, t->ngiven, t->ldxgiven, t->xgiven, t->fgiven);
-    }
-  }
-}
-
-/*********************************************************************/
-
-#include "common.c"
-
 Extern void EXPORT(Divonne)(ccount ndim, ccount ncomp,
   Integrand integrand, void *userdata,
   creal epsrel, creal epsabs,
@@ -68,7 +22,7 @@ Extern void EXPORT(Divonne)(ccount ndim, ccount ncomp,
   cnumber mineval, cnumber maxeval,
   cint key1, cint key2, cint key3, ccount maxpass,
   creal border, creal maxchisq, creal mindeviation,
-  cnumber ngiven, ccount ldxgiven, creal *xgiven,
+  cnumber ngiven, ccount ldxgiven, real *xgiven,
   cnumber nextra, PeakFinder peakfinder,
   int *pnregions, number *pneval, int *pfail,
   real *integral, real *error, real *prob)
@@ -92,20 +46,16 @@ Extern void EXPORT(Divonne)(ccount ndim, ccount ncomp,
   t.maxchisq = maxchisq;
   t.mindeviation = mindeviation;
   t.ngiven = ngiven;
-  t.xgiven = NULL;
+  t.xgiven = xgiven;
   t.ldxgiven = ldxgiven;
   t.nextra = nextra;
   t.peakfinder = peakfinder;
   t.nregions = 0;
   t.neval = 0;
 
-  AllocGiven(&t, xgiven);
-
   *pfail = Integrate(&t, integral, error, prob);
   *pnregions = t.nregions;
   *pneval = t.neval;
-
-  free(t.xgiven);
 }
 
 /*********************************************************************/
@@ -117,7 +67,7 @@ Extern void EXPORT(divonne)(ccount *pndim, ccount *pncomp,
   cnumber *pmineval, cnumber *pmaxeval,
   cint *pkey1, cint *pkey2, cint *pkey3, ccount *pmaxpass,
   creal *pborder, creal *pmaxchisq, creal *pmindeviation,
-  cnumber *pngiven, ccount *pldxgiven, creal *xgiven,
+  cnumber *pngiven, ccount *pldxgiven, real *xgiven,
   cnumber *pnextra, PeakFinder peakfinder,
   int *pnregions, number *pneval, int *pfail,
   real *integral, real *error, real *prob)
@@ -141,19 +91,15 @@ Extern void EXPORT(divonne)(ccount *pndim, ccount *pncomp,
   t.maxchisq = *pmaxchisq;
   t.mindeviation = *pmindeviation;
   t.ngiven = *pngiven;
-  t.xgiven = NULL;
+  t.xgiven = xgiven;
   t.ldxgiven = *pldxgiven;
   t.nextra = *pnextra;
   t.peakfinder = peakfinder;
   t.nregions = 0;
   t.neval = 0;
 
-  AllocGiven(&t, xgiven);
-
   *pfail = Integrate(&t, integral, error, prob);
   *pnregions = t.nregions;
   *pneval = t.neval;
-
-  free(t.xgiven);
 }
 
