@@ -4,25 +4,20 @@
 		has approximately equal spread = 1/2 vol (max - min),
 		then do a main integration over all regions
 		this file is part of Divonne
-		last modified 8 Jun 10 th
+		last modified 28 Mar 13 th
 */
 
 
-#define INIDEPTH 3
-#define DEPTH 5
-#define POSTDEPTH 15
-
-/*********************************************************************/
-
 static int Integrate(This *t, real *integral, real *error, real *prob)
 {
   TYPEDEFREGION;
 
   Totals totals[NCOMP];
-  real nneed, weight;
+  real nneed;
   count dim, comp, iter, pass = 0, err, iregion;
   number nwant, nmin = INT_MAX;
-  int fail = -99;
+  ML_ONLY(number neff;)
+  int fail;
 
   if( VERBOSE > 1 ) {
     char s[512];
@@ -33,7 +28,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       "  mineval " NUMBER "\n  maxeval " NUMBER "\n"
       "  key1 %d\n  key2 %d\n  key3 %d\n  maxpass " COUNT "\n"
       "  border " REAL "\n  maxchisq " REAL "\n  mindeviation " REAL "\n"
-      "  ngiven " NUMBER "\n  nextra " NUMBER "\n",
+      "  ngiven " NUMBER "\n  nextra " NUMBER,
       t->ndim, t->ncomp,
       t->epsrel, t->epsabs,
       t->flags, t->seed,
@@ -49,25 +44,28 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       BadDimension(t, t->key2) ||   
       ((t->key3 & -2) && BadDimension(t, t->key3)) ) return -1;
 
+  FORK_ONLY(t->nframe = 0;)
   t->neval_opt = t->neval_cut = 0;
 
-  t->size = CHUNKSIZE;
-  MemAlloc(t->voidregion, t->size*sizeof(Region));
+  AllocGiven(t);
+  AllocRegions(t);
+  SamplesIni(&t->samples[0]);
+  SamplesIni(&t->samples[1]);
+  SamplesIni(&t->samples[2]);
+  RuleAlloc(t);
+  if( IsSobol(t->key1) || IsSobol(t->key2) || IsSobol(t->key3) )
+    IniRandom(t);
+
+  ForkCores(t);
+
   for( dim = 0; dim < t->ndim; ++dim ) {
     Bounds *b = &RegionPtr(0)->bounds[dim];
     b->lower = 0;
     b->upper = 1;
   }
+  t->nregions = 1;
 
-  RuleIni(&t->rule7);
-  RuleIni(&t->rule9);
-  RuleIni(&t->rule11);
-  RuleIni(&t->rule13);
-  SamplesIni(&t->samples[0]);
-  SamplesIni(&t->samples[1]);
-  SamplesIni(&t->samples[2]);
-
-  if( setjmp(t->abort) ) goto abort;
+  if( (fail = setjmp(t->abort)) ) goto abort;
 
   t->epsabs = Max(t->epsabs, NOTZERO);
 
@@ -75,9 +73,6 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
   if( VERBOSE ) Print("Partitioning phase:");
 
-  if( IsSobol(t->key1) || IsSobol(t->key2) || IsSobol(t->key3) )
-    IniRandom(t);
-
   SamplesLookup(t, &t->samples[0], t->key1,
     (number)47, (number)INT_MAX, (number)0);
   SamplesAlloc(t, &t->samples[0]);
@@ -86,7 +81,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   Zap(totals);
   t->phase = 1;
 
-  Explore(t, 0, &t->samples[0], INIDEPTH, 1);
+  Iterate(t, 0, INIDEPTH, 0, NULL);
 
   for( iter = 1; ; ++iter ) {
     Totals *maxtot;
@@ -123,7 +118,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       valid += tot->avg == tot->avg;
       if( tot->spreadsq > maxtot->spreadsq ) maxtot = tot;
       tot->spread = sqrt(tot->spreadsq);
-      error[comp] = tot->spread*t->samples[0].weight;
+      error[comp] = tot->spread/t->samples[0].neff;
     }
 
     if( VERBOSE ) {
@@ -158,7 +153,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       else if( ++pass > t->maxpass && n >= t->mineval ) break;
     }
 
-    Split(t, maxtot->iregion, DEPTH);
+    Iterate(t, maxtot->iregion, DEPTH, -1, NULL);
   }
 
   /* Step 2: do a "full" integration on each region */
@@ -185,7 +180,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     if( VERBOSE ) Print("\nNot enough samples left for main integration.");
     for( comp = 0; comp < t->ncomp; ++comp )
       prob[comp] = -999;
-    weight = t->samples[0].weight;
+    ML_ONLY(neff = t->samples[0].neff;)
   }
   else {
     bool can_adjust = (t->key3 == 1 && t->samples[1].sampler != SampleRule &&
@@ -202,21 +197,36 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       Print(s);
     }
 
-    ResClear(integral);
-    ResClear(error);
-    ResClear(prob);
+    FClear(integral);
+    FClear(error);
+    FClear(prob);
 
     nlimit = t->maxeval - t->nregions*t->samples[1].n;
     df = 0;
 
+#define CopyPhaseResults(f) \
+  for( comp = 0; comp < t->ncomp; ++comp ) { \
+    PhaseResult *p = &totals[comp].phase[f]; \
+    cResult *r = &region->result[comp]; \
+    p->avg = r->avg; \
+    p->err = r->err; \
+  }
+
+#define Var2(f, res) Sq((res)->err ? (res)->err : r->spread/t->samples[f].neff)
+#define Var(f) Var2(f, &tot->phase[f])
+
     for( iregion = 0; iregion < t->nregions; ++iregion ) {
-      Region *region = RegionPtr(iregion);
+      Region *region;
       char s[64*NDIM + 256*NCOMP], *p = s;
       int todo;
 
 refine:
+      region = RegionPtr(iregion);
+      CopyPhaseResults(0);
       t->phase = 2;
-      t->samples[1].sampler(t, &t->samples[1], region->bounds, region->vol);
+      region->isamples = 1;
+      t->samples[1].sampler(t, iregion);
+      CopyPhaseResults(1);
 
       if( can_adjust )
         for( comp = 0; comp < t->ncomp; ++comp )
@@ -229,41 +239,20 @@ refine:
         cResult *r = &region->result[comp];
         Totals *tot = &totals[comp];
 
-        t->samples[0].avg[comp] = r->avg;
-        t->samples[0].err[comp] = r->err;
-
         if( t->neval < nlimit ) {
-          creal avg2 = t->samples[1].avg[comp];
-          creal err2 = t->samples[1].err[comp];
-          creal diffsq = Sq(avg2 - r->avg);
+          creal avg2 = tot->phase[1].avg;
+          creal diffsq = Sq(avg2 - tot->phase[0].avg);
 
-#define Var(s) Sq((s.err[comp] == 0) ? r->spread*s.weight : s.err[comp])
-
-          if( err2*tot->nneed > r->spread ||
-              diffsq > Max(t->maxchisq*(Var(t->samples[0]) + Var(t->samples[1])),
-                           EPS*Sq(avg2)) ) {
+          if( r->err*tot->nneed > r->spread ||
+              diffsq > Max(t->maxchisq*(Var(0) + Var(1)), EPS*Sq(avg2)) ) {
             if( t->key3 && diffsq > tot->mindevsq ) {
               if( t->key3 == 1 ) {
-                ccount xregion = t->nregions;
-
                 if( VERBOSE > 2 ) Print("\nSplit");
-
                 t->phase = 1;
-                Explore(t, iregion, &t->samples[1], POSTDEPTH, 2);
+                Iterate(t, iregion, POSTDEPTH, 1, totals);
 
                 if( can_adjust ) {
-                  number nnew;
-                  count ireg, xreg;
-
-                  for( ireg = iregion, xreg = xregion;
-                       ireg < t->nregions; ireg = xreg++ ) {
-                    cResult *result = RegionPtr(ireg)->result;
-                    count c;
-                    for( c = 0; c < t->ncomp; ++c )
-                      totals[c].spreadsq += Sq(result[c].spread);
-                  }
-
-                  nnew = (tot->spreadsq/Sq(MARKMASK) > tot->maxerrsq) ?
+                  cnumber nnew = (tot->spreadsq/Sq(MARKMASK) > tot->maxerrsq) ?
                     MARKMASK :
                     (number)ceil(sqrt(tot->spreadsq/tot->maxerrsq));
                   if( nnew > nwant + nwant/64 ) {
@@ -286,7 +275,6 @@ refine:
                     }
                   }
                 }
-
                 goto refine;
               }
               todo |= 3;
@@ -299,12 +287,13 @@ refine:
       if( can_adjust ) {
         for( comp = 0; comp < t->ncomp; ++comp )
           totals[comp].maxerrsq -=
-            Sq(region->result[comp].spread*t->samples[1].weight);
+            Sq(region->result[comp].spread/t->samples[1].neff);
       }
 
       switch( todo ) {
       case 1:	/* get spread right */
-        Explore(t, iregion, &t->samples[1], 0, 2);
+        region->isamples = 1;
+        ExploreSerial(t, iregion);
         break;
 
       case 3:	/* sample region again with more points */
@@ -314,14 +303,13 @@ refine:
           SamplesAlloc(t, &t->samples[2]);
         }
         t->phase = 3;
-        t->samples[2].sampler(t, &t->samples[2], region->bounds, region->vol);
-        Explore(t, iregion, &t->samples[2], 0, 2);
+        region->isamples = 2;
+        t->samples[2].sampler(t, iregion);
+        ExploreSerial(t, iregion);
         ++region->depth;	/* misused for df here */
         ++df;
       }
 
-      ++region->depth;	/* misused for df here */
-
       if( VERBOSE > 2 ) {
         for( dim = 0; dim < t->ndim; ++dim ) {
           cBounds *b = &region->bounds[dim];
@@ -334,44 +322,45 @@ refine:
 
       for( comp = 0; comp < t->ncomp; ++comp ) {
         Result *r = &region->result[comp];
+        Totals *tot = &totals[comp];
 
-        creal x1 = t->samples[0].avg[comp];
-        creal s1 = Var(t->samples[0]);
-        creal x2 = t->samples[1].avg[comp];
-        creal s2 = Var(t->samples[1]);
-        creal r2 = (s1 == 0) ? Sq(t->samples[1].neff*t->samples[0].weight) : s2/s1;
+        creal x1 = tot->phase[0].avg;
+        creal v1 = Var(0);
+        creal x2 = tot->phase[1].avg;
+        creal v2 = Var(1);
+        creal r2 = v1 ? v2/v1 :
+          Sq(t->samples[1].neff/(real)t->samples[0].neff);
 
         real norm = 1 + r2;
         real avg = x2 + r2*x1;
-        real sigsq = s2;
+        real sigsq = v2;
         real chisq = Sq(x2 - x1);
-        real chiden = s1 + s2;
+        real chiden = v1 + v2;
 
         if( todo == 3 ) {
-          creal x3 = t->samples[2].avg[comp];
-          creal s3 = Var(t->samples[2]);
-          creal r3 = (s2 == 0) ? Sq(t->samples[2].neff*t->samples[1].weight) : s3/s2;
+          creal x3 = r->avg;
+          creal v3 = Var2(2, r);
+          creal r3 = v2 ? v3/v2 :
+            Sq(t->samples[2].neff/(real)t->samples[1].neff);
 
           norm = 1 + r3*norm;
           avg = x3 + r3*avg;
-          sigsq = s3;
-          chisq = s1*Sq(x3 - x2) + s2*Sq(x3 - x1) + s3*chisq;
-          chiden = s1*s2 + s3*chiden;
+          sigsq = v3;
+          chisq = v1*Sq(x3 - x2) + v2*Sq(x3 - x1) + v3*chisq;
+          chiden = v1*v2 + v3*chiden;
         }
 
         avg = LAST ? r->avg : (sigsq *= norm = 1/norm, avg*norm);
         if( chisq > EPS ) chisq /= Max(chiden, NOTZERO);
 
-#define Out(s) s.avg[comp], r->spread*s.weight, s.err[comp]
-
         if( VERBOSE > 2 ) {
+#define Out2(f, res) (res)->avg, r->spread/t->samples[f].neff, (res)->err
+#define Out(f) Out2(f, &tot->phase[f])
           p += sprintf(p, "\n[" COUNT "] "
             REAL " +- " REAL "(" REAL ")\n    "
-            REAL " +- " REAL "(" REAL ")",
-            comp + 1, Out(t->samples[0]), Out(t->samples[1]));
+            REAL " +- " REAL "(" REAL ")", comp + 1, Out(0), Out(1));
           if( todo == 3 ) p += sprintf(p, "\n    "
-            REAL " +- " REAL "(" REAL ")",
-            Out(t->samples[2]));
+            REAL " +- " REAL "(" REAL ")", Out2(2, r));
           p += sprintf(p, "  \tchisq " REAL, chisq);
         }
 
@@ -408,7 +397,7 @@ refine:
     for( comp = 0; comp < t->ncomp; ++comp )
       prob[comp] = ChiSquare(prob[comp], df);
 
-    weight = 1;
+    ML_ONLY(neff = 1;)
   }
 
 #ifdef MLVERSION
@@ -433,25 +422,25 @@ refine:
       MLPutFunction(stdlink, "List", t->ncomp);
       for( comp = 0; comp < t->ncomp; ++comp ) {
         cResult *r = &region->result[comp];
-        real res[] = {r->avg, r->spread*weight, r->chisq};
+        real res[] = {r->avg, r->spread/neff, r->chisq};
         MLPutRealList(stdlink, res, Elements(res));
       }
 
-      MLPutInteger(stdlink, region->depth);  /* misused for df */
+      MLPutInteger(stdlink, region->depth + 1);  /* misused for df */
     }
   }
 #endif
 
 abort:
+  WaitCores(t);
+  FORK_ONLY(FrameFree(t, ShmRm(t));)
+
+  RuleFree(t);
   SamplesFree(&t->samples[2]);
   SamplesFree(&t->samples[1]);
   SamplesFree(&t->samples[0]);
-  RuleFree(&t->rule13);
-  RuleFree(&t->rule11);
-  RuleFree(&t->rule9);
-  RuleFree(&t->rule7);
-
   free(t->voidregion);
+  free(t->xgiven);
 
   return fail;
 }
