@@ -2,7 +2,7 @@
 	Integrate.c
 		integrate over the unit hypercube
 		this file is part of Suave
-		last modified 16 Jun 10 th
+		last modified 10 Mar 13 th
 */
 
 
@@ -11,7 +11,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   TYPEDEFREGION;
 
   count dim, comp, df;
-  int fail = -99;
+  int fail;
   Result totals[NCOMP];
   Region *anchor = NULL, *region = NULL;
 
@@ -34,12 +34,15 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   if( BadComponent(t) ) return -2;
   if( BadDimension(t) ) return -1;
 
-  if( setjmp(t->abort) ) goto abort;
+  ShmAlloc(t, ShmRm(t));
+  ForkCores(t);
+
+  if( (fail = setjmp(t->abort)) ) goto abort;
 
   t->epsabs = Max(t->epsabs, NOTZERO);
   IniRandom(t);
 
-  RegionAlloc(anchor, t->nnew, t->nnew);
+  RegionAlloc(t, anchor, t->nnew, t->nnew);
   anchor->next = NULL;
   anchor->div = 0;
 
@@ -62,7 +65,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     anchor->w + t->nnew,
     anchor->w + t->nnew + t->ndim*t->nnew);
   df = anchor->df;
-  ResCopy(totals, anchor->result);
+  FCopy(totals, anchor->result);
 
   for( t->nregions = 1; ; ++t->nregions ) {
     Var var[NDIM][2], *vLR;
@@ -100,13 +103,13 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       }
     }
 
-    if( maxratio <= 1 && t->neval >= t->mineval ) {
-      fail = 0;
+    if( maxratio <= 1 && t->neval >= t->mineval ) break;
+
+    if( t->neval >= t->maxeval ) {
+      fail = 1;
       break;
     }
 
-    if( t->neval >= t->maxeval ) break;
-
     maxerr = -INFTY;
     parent = &anchor;
     region = anchor;
@@ -146,8 +149,8 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     nnewR = IMax(t->nnew - nnewL, MINSAMPLES);
     nR = vLR[1].n + nnewR;
 
-    RegionAlloc(regionL, nL, nnewL);
-    RegionAlloc(regionR, nR, nnewR);
+    RegionAlloc(t, regionL, nL, nnewL);
+    RegionAlloc(t, regionR, nR, nnewR);
 
     *parent = regionL;
     regionL->next = regionR;
@@ -164,19 +167,19 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     while( n-- ) {
       cbool final = (*w < 0);
       if( x[bisectdim] < mid ) {
-        if( final && wR > regionR->w ) *(wR - 1) = -fabs(*(wR - 1));
+        if( final && wR > regionR->w ) wR[-1] = -fabs(wR[-1]);
         *wL++ = *w++;
-        VecCopy(xL, x);
+        XCopy(xL, x);
         xL += t->ndim;
-        ResCopy(fL, f);
+        FCopy(fL, f);
         fL += t->ncomp;
       }
       else {
-        if( final && wL > regionL->w ) *(wL - 1) = -fabs(*(wL - 1));
+        if( final && wL > regionL->w ) wL[-1] = -fabs(wL[-1]);
         *wR++ = *w++;
-        VecCopy(xR, x);
+        XCopy(xR, x);
         xR += t->ndim;
-        ResCopy(fR, f);
+        FCopy(fR, f);
         fR += t->ncomp;
       }
       x += t->ndim;
@@ -185,8 +188,8 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     }
 
     Reweight(t, region->bounds, wlast, flast, f, totals);
-    VecCopy(regionL->bounds, region->bounds);
-    VecCopy(regionR->bounds, region->bounds);
+    XCopy(regionL->bounds, region->bounds);
+    XCopy(regionR->bounds, region->bounds);
 
     boundsL = &regionL->bounds[bisectdim];
     boundsR = &regionR->bounds[bisectdim];
@@ -267,11 +270,12 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
 abort:
   free(region);
-
   while( (region = anchor) ) {
     anchor = anchor->next;
     free(region);
   }
+  WaitCores(t);
+  ShmFree(t);
 
   return fail;
 }
