@@ -2,7 +2,7 @@
 	decl.h
 		Type declarations
 		this file is part of Suave
-		last modified 6 Jun 10 th
+		last modified 21 Dec 11 th
 */
 
 
@@ -34,13 +34,19 @@ typedef struct {
 
 typedef const Bounds cBounds;
 
-typedef int (*Integrand)(ccount *, creal *, ccount *, real *, void *, creal *);
+typedef int (*Integrand)(ccount *, creal *, ccount *, real *,
+  void *, creal *, cint *);
 
 typedef struct _this {
   count ndim, ncomp;
 #ifndef MLVERSION
   Integrand integrand;
   void *userdata;
+#ifdef HAVE_FORK
+  int ncores, *child;
+  real *frame;
+  SHM_ONLY(int shmid;)
+#endif
 #endif
   real epsrel, epsabs;
   int flags, seed;
@@ -53,6 +59,8 @@ typedef struct _this {
   jmp_buf abort;
 } This;
 
+#define nframe nnew
+
 typedef const This cThis;
 
 #define TYPEDEFREGION \
