@@ -2,15 +2,15 @@
 	Integrate.c
 		integrate over the unit hypercube
 		this file is part of Vegas
-		last modified 8 Jun 10 th
+		last modified 17 Apr 12 th
 */
 
 
 static int Integrate(This *t, real *integral, real *error, real *prob)
 {
-  real *sample;
+  bin_t *bins;
   count dim, comp;
-  int fail = -99;
+  int fail;
   struct {
     count niter;
     number nsamples, neval;
@@ -29,7 +29,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       "  mineval " NUMBER "\n  maxeval " NUMBER "\n"
       "  nstart " NUMBER "\n  nincrease " NUMBER "\n"
       "  nbatch " NUMBER "\n  gridno %d\n"
-      "  statefile \"%s\"\n",
+      "  statefile \"%s\"",
       t->ndim, t->ncomp,
       t->epsrel, t->epsabs,
       t->flags, t->seed,
@@ -42,29 +42,31 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   if( BadComponent(t) ) return -2;
   if( BadDimension(t) ) return -1;
 
-  SamplesAlloc(sample);
+  FrameAlloc(t, ShmRm(t));
+  ForkCores(t);
+  Alloc(bins, t->nbatch*t->ndim);
 
-  if( setjmp(t->abort) ) goto abort;
+  if( (fail = setjmp(t->abort)) ) goto abort;
 
   IniRandom(t);
 
-  if( t->statefile && *t->statefile ) {
-    if( stat(t->statefile, &st) == 0 &&
-        st.st_size == sizeof(state) && (st.st_mode & 0400) ) {
-      cint h = open(t->statefile, O_RDONLY);
-      read(h, &state, sizeof(state));
-      close(h);
-      t->rng.skiprandom(t, t->neval = state.neval);
+  if( t->statefile && *t->statefile == 0 ) t->statefile = NULL;
 
-      if( VERBOSE ) {
-        char s[256];
-        sprintf(s, "\nRestoring state from %s.", t->statefile);
-        Print(s);
-      }
+  if( t->statefile &&
+      stat(t->statefile, &st) == 0 &&
+      st.st_size == sizeof state && (st.st_mode & 0400) ) {
+    cint h = open(t->statefile, O_RDONLY);
+    read(h, &state, sizeof state);
+    close(h);
+    t->rng.skiprandom(t, t->neval = state.neval);
+
+    if( VERBOSE ) {
+      char s[256];
+      sprintf(s, "\nRestoring state from %s.", t->statefile);
+      Print(s);
     }
   }
   else {
-    t->statefile = NULL;
     state.niter = 0;
     state.nsamples = t->nstart;
     Zap(state.cumul);
@@ -82,11 +84,11 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
 
     for( ; nsamples > 0; nsamples -= t->nbatch ) {
       cnumber n = IMin(t->nbatch, nsamples);
-      real *w = sample;
+      real *w = t->frame;
       real *x = w + n;
       real *f = x + n*t->ndim;
       real *lastf = f + n*t->ncomp;
-      bin_t *bin = (bin_t *)lastf;
+      bin_t *bin = bins;
 
       while( x < f ) {
         real weight = jacobian;
@@ -106,10 +108,10 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
         *w++ = weight;
       }
 
-      DoSample(t, n, sample, w, f);
+      DoSample(t, n, w, f, t->frame, state.niter + 1);
 
-      w = sample;
-      bin = (bin_t *)lastf;
+      bin = bins;
+      w = t->frame;
 
       while( f < lastf ) {
         creal weight = *w++;
@@ -175,7 +177,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
     }
 
     if( fail == 0 && t->neval >= t->mineval ) {
-      if( t->statefile ) unlink(t->statefile);
+      if( t->statefile && KEEPFILE == 0 ) unlink(t->statefile);
       break;
     }
 
@@ -209,7 +211,7 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
       cint h = creat(t->statefile, 0666);
       if( h != -1 ) {
         state.neval = t->neval;
-        write(h, &state, sizeof(state));
+        write(h, &state, sizeof state);
         close(h);
 
         if( statemsg ) {
@@ -231,8 +233,10 @@ static int Integrate(This *t, real *integral, real *error, real *prob)
   }
 
 abort:
-  free(sample);
   PutGrid(t, state.grid);
+  free(bins);
+  WaitCores(t);
+  FrameFree(t);
 
   return fail;
 }
