@@ -2,31 +2,18 @@
 	Vegas.c
 		Vegas Monte-Carlo integration
 		by Thomas Hahn
-		last modified 16 Jun 10 th
+		last modified 19 Dec 11 th
 */
 
 
+#define VEGAS
+#define ROUTINE "Vegas"
+
 #include "decl.h"
-
-#define Print(s) puts(s); fflush(stdout)
+#include "CSample.c"
 
 /*********************************************************************/
 
-static inline void DoSample(This *t, number n, creal *w, creal *x, real *f)
-{
-  t->neval += n;
-  while( n-- ) {
-    if( t->integrand(&t->ndim, x, &t->ncomp, f, t->userdata, w++) == ABORT )
-      longjmp(t->abort, 1);
-    x += t->ndim;
-    f += t->ncomp;
-  }
-}
-
-/*********************************************************************/
-
-#include "common.c"
-
 Extern void EXPORT(Vegas)(ccount ndim, ccount ncomp,
   Integrand integrand, void *userdata,
   creal epsrel, creal epsabs, cint flags, cint seed,
