@@ -2,7 +2,7 @@
 	decl.h
 		Type declarations
 		this file is part of Vegas
-		last modified 6 Jun 10 th
+		last modified 21 Dec 11 th
 */
 
 
@@ -28,14 +28,20 @@ typedef struct {
 
 typedef const Cumulants cCumulants;
 
-typedef int (*Integrand)(ccount *, creal *, ccount *, real *, void *, creal *);
+typedef int (*Integrand)(ccount *, creal *, ccount *, real *,
+  void *, creal *, cint *);
 
 typedef struct _this {
   count ndim, ncomp;
 #ifndef MLVERSION
   Integrand integrand;
   void *userdata;
+#ifdef HAVE_FORK
+  int ncores, *child;
+  SHM_ONLY(int shmid;)
 #endif
+#endif
+  real *frame;
   real epsrel, epsabs;
   int flags, seed;
   number mineval, maxeval;
@@ -47,6 +53,8 @@ typedef struct _this {
   jmp_buf abort;
 } This;
 
+#define nframe nbatch
+
 typedef const This cThis;
 
 static Grid *gridptr_[MAXGRIDS];
